#include <SoftwareSerial.h>
SoftwareSerial GSM(10, 11); // RX, TX
uint8_t LED1 = 3 ; // Connection Yellow LED
uint8_t LED2 = 4 ; // Connection Red LED
uint8_t LED3 = 5 ; // Connection Blue LED 
uint8_t LED4 = 6 ; // Connection Green LED 
uint8_t LEDGSM = 7 ; // Connection Green LED 
int serverstate;
 
enum _parseState {
  PS_DETECT_MSG_TYPE,
  PS_IGNORING_COMMAND_ECHO,
  PS_HTTPACTION_TYPE,
  PS_HTTPACTION_RESULT,
  PS_HTTPACTION_LENGTH,
  PS_HTTPREAD_LENGTH,
  PS_HTTPREAD_CONTENT
};
enum _actionState {
  AS_IDLE,
  AS_WAITING_FOR_RESPONSE
};
byte actionState = AS_IDLE;
unsigned long lastActionTime = 0;
byte parseState = PS_DETECT_MSG_TYPE;
char buffer[80];
byte pos = 0;
int contentLength = 0;

void resetBuffer() {
  memset(buffer, 0, sizeof(buffer));
  pos = 0;
}

void sendGSM(const char* msg, int waitMs = 1500) {
  GSM.println(msg);
  while(GSM.available()) {
    parseATText(GSM.read());
  }
  delay(waitMs);
}

void setup()
{
  GSM.begin(9600);
  Serial.begin(9600);
  pinMode (LED1, OUTPUT);
  pinMode (LED2, OUTPUT);
  pinMode (LED3, OUTPUT);  
  pinMode (LED4, OUTPUT);
  pinMode (LED4, OUTPUT); 
  pinMode (LEDGSM, OUTPUT); 
  delay(15000);    
  sendGSM("AT+SAPBR=3,1,\"APN\",\"Kainak-IOT\"");  
  sendGSM("AT+SAPBR=1,1",3000);
  sendGSM("AT+HTTPINIT");  
  sendGSM("AT+HTTPPARA=\"CID\",1");
  sendGSM("AT+HTTPPARA=\"URL\",\"ipsa99.ir/relays.php/?sp=MTAwMDAxMTEwMDEwMTEwMTEwMTEwMTAxMTAxMTExMDAwMDAxMTA=\"");
}

void loop()
{ 
  unsigned long now = millis();
  if ( actionState == AS_IDLE ) {
    if ( now > lastActionTime + 1500 ) {
      digitalWrite (LEDGSM,0);
      sendGSM("AT+HTTPINIT");
      sendGSM("AT+HTTPPARA=\"CID\",1");
      sendGSM("AT+HTTPPARA=\"URL\",\"ipsa99.ir/relays.php/?sp=MTAwMDAxMTEwMDEwMTEwMTEwMTEwMTAxMTAxMTExMDAwMDAxMTA=\"",100);
      sendGSM("AT+HTTPACTION=0");
      lastActionTime = now;
      actionState = AS_WAITING_FOR_RESPONSE;
      digitalWrite (LEDGSM,1);
    }
  }
  while(GSM.available()) {
    lastActionTime = now;
    parseATText(GSM.read());
  }
}

void parseATText(byte b) {
  buffer[pos++] = b;
  if ( pos >= sizeof(buffer) )
    resetBuffer(); // just to be safe
  
  switch (parseState) {
  case PS_DETECT_MSG_TYPE: 
    {
      if ( b == '\n' )
        resetBuffer();
      else {        
        if ( pos == 3 && strcmp(buffer, "AT+") == 0 ) {
          parseState = PS_IGNORING_COMMAND_ECHO;
        }
        else if ( b == ':' ) {
          //Serial.print("Checking message type: ");
          //Serial.println(buffer);

          if ( strcmp(buffer, "+HTTPACTION:") == 0 ) {
            Serial.println("Received HTTPACTION");
            parseState = PS_HTTPACTION_TYPE;
          }
          else if ( strcmp(buffer, "+HTTPREAD:") == 0 ) {
            Serial.println("Received HTTPREAD");            
            parseState = PS_HTTPREAD_LENGTH;
          }
          resetBuffer();
        }
      }
    }
    break;


  case PS_IGNORING_COMMAND_ECHO:
    {
      if ( b == '\n' ) {
        Serial.print("AT Command: ");
        Serial.println(buffer);
        parseState = PS_DETECT_MSG_TYPE;
        resetBuffer();
      }
    }
    break;

  case PS_HTTPACTION_TYPE:
    {
      if ( b == ',' ) {
        Serial.print("HTTPACTION type is ");
        Serial.println(buffer);
        parseState = PS_HTTPACTION_RESULT;
        resetBuffer();
      }
    }
    break;

  case PS_HTTPACTION_RESULT:
    {
      if ( b == ',' ) {
        Serial.print("HTTPACTION result is ");
        Serial.println(buffer);
        parseState = PS_HTTPACTION_LENGTH;
        resetBuffer();
      }
    }
    break;

  case PS_HTTPACTION_LENGTH:
    {
      if ( b == '\n' ) {
        Serial.print("HTTPACTION length is ");
        Serial.println(buffer);
        
        // now request content
        GSM.print("AT+HTTPREAD=0,");
        GSM.println(buffer);
             
        parseState = PS_DETECT_MSG_TYPE;
        resetBuffer();
      }
    }
    break;

  case PS_HTTPREAD_LENGTH:
    {
      if ( b == '\n' ) {
        contentLength = atoi(buffer);
        Serial.print("HTTPREAD length is ");
        Serial.println(contentLength);
        
        Serial.print("HTTPREAD content: ");     
        parseState = PS_HTTPREAD_CONTENT;      
        resetBuffer();
      }
    }
    break;

  case PS_HTTPREAD_CONTENT:
    {
      // for this demo I'm just showing the content bytes in the serial monitor
      Serial.write(b);
      contentLength--;
      if ( contentLength <= 0 ) {
        // all content bytes have now been read
        parseState = PS_DETECT_MSG_TYPE;
        //serverstate=atoi(buffer);
        //------------Led1------------->
        //----------------------------->
        if(buffer[0]=='1'){
            digitalWrite (LED1,1);
        } 
        if(buffer[0]=='0'){
            digitalWrite (LED1,0);
        } 
        //------------Led2------------->
        //----------------------------->
        if(buffer[1]=='1'){
            digitalWrite (LED2,1);
        } 
        if(buffer[1]=='0'){
            digitalWrite (LED2,0);
        } 
        //------------Led3------------->
        //----------------------------->
        if(buffer[2]=='1'){
            digitalWrite (LED3,1);
        } 
        if(buffer[2]=='0'){
            digitalWrite (LED3,0);
        }    
        //------------Led4------------->
        //----------------------------->
        if(buffer[3]=='1'){
            digitalWrite (LED4,1);
        } 
        if(buffer[3]=='0'){
            digitalWrite (LED4,0);
        } 
        resetBuffer();
        Serial.print("\n\n\n");
        actionState = AS_IDLE;
      }
    }
    break;
  }
}
